@@ -2,18 +2,16 @@ package sriovnetworknodepolicy
 
 import (
 	"context"
-	"crypto/md5"
-	"encoding/hex"
+
 	"encoding/json"
 	"fmt"
 	"os"
 	"reflect"
 	"sort"
-	// "time"
+	"time"
 
 	dptypes "github.com/intel/sriov-network-device-plugin/pkg/types"
 	errs "github.com/pkg/errors"
-	admissionregistrationv1beta1 "k8s.io/api/admissionregistration/v1beta1"
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
 	rbacv1 "k8s.io/api/rbac/v1"
@@ -27,12 +25,13 @@ import (
 	"sigs.k8s.io/controller-runtime/pkg/controller"
 	"sigs.k8s.io/controller-runtime/pkg/controller/controllerutil"
 	"sigs.k8s.io/controller-runtime/pkg/handler"
+	logf "sigs.k8s.io/controller-runtime/pkg/log"
 	"sigs.k8s.io/controller-runtime/pkg/manager"
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
-	logf "sigs.k8s.io/controller-runtime/pkg/runtime/log"
 	"sigs.k8s.io/controller-runtime/pkg/source"
 
 	sriovnetworkv1 "github.com/openshift/sriov-network-operator/pkg/apis/sriovnetwork/v1"
+	"github.com/openshift/sriov-network-operator/pkg/controller/sriovoperatorconfig"
 	render "github.com/openshift/sriov-network-operator/pkg/render"
 )
 
@@ -41,14 +40,12 @@ var log = logf.Log.WithName("controller_sriovnetworknodepolicy")
 // ManifestPaths is the path to the manifest templates
 // bad, but there's no way to pass configuration to the reconciler right now
 const (
-	PLUGIN_PATH                 = "./bindata/manifests/plugins"
-	DAEMON_PATH                 = "./bindata/manifests/daemon"
-	WEBHOOK_PATH                = "./bindata/manifests/webhook"
-	DEFAULT_POLICY_NAME         = "default"
-	CONFIGMAP_NAME              = "device-plugin-config"
-	DP_CONFIG_FILENAME          = "config.json"
-	SERVICE_CA_CONFIGMAP        = "openshift-service-ca"
-	SRIOV_MUTATING_WEBHOOK_NAME = "network-resources-injector-config"
+	ResyncPeriod        = 5 * time.Minute
+	PLUGIN_PATH         = "./bindata/manifests/plugins"
+	DAEMON_PATH         = "./bindata/manifests/daemon"
+	DEFAULT_POLICY_NAME = "default"
+	CONFIGMAP_NAME      = "device-plugin-config"
+	DP_CONFIG_FILENAME  = "config.json"
 )
 
 var Namespace = os.Getenv("NAMESPACE")
@@ -83,71 +80,17 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 		IsController: true,
 		OwnerType:    &sriovnetworkv1.SriovNetworkNodePolicy{},
 	})
-
 	if err != nil {
 		return err
 	}
 
-	// Watch for changes to secondary resource ServiceAccount
-	err = c.Watch(&source.Kind{Type: &corev1.ServiceAccount{}}, &handler.EnqueueRequestForOwner{
+	err = c.Watch(&source.Kind{Type: &appsv1.DaemonSet{}}, &handler.EnqueueRequestForOwner{
 		IsController: true,
-		OwnerType:    &sriovnetworkv1.SriovNetworkNodePolicy{},
+		OwnerType:    &sriovnetworkv1.SriovOperatorConfig{},
 	})
-
 	if err != nil {
 		return err
 	}
-
-	// Watch for changes to secondary resource ConfigMap
-	err = c.Watch(&source.Kind{Type: &corev1.ConfigMap{}}, &handler.EnqueueRequestForOwner{
-		IsController: true,
-		OwnerType:    &sriovnetworkv1.SriovNetworkNodePolicy{},
-	})
-
-	if err != nil {
-		return err
-	}
-
-	// Watch for changes to secondary resource ClusterRole
-	err = c.Watch(&source.Kind{Type: &rbacv1.ClusterRole{}}, &handler.EnqueueRequestForOwner{
-		IsController: true,
-		OwnerType:    &sriovnetworkv1.SriovNetworkNodePolicy{},
-	})
-
-	if err != nil {
-		return err
-	}
-
-	// Watch for changes to secondary resource ClusterRoleBinding
-	err = c.Watch(&source.Kind{Type: &rbacv1.ClusterRoleBinding{}}, &handler.EnqueueRequestForOwner{
-		IsController: true,
-		OwnerType:    &sriovnetworkv1.SriovNetworkNodePolicy{},
-	})
-
-	if err != nil {
-		return err
-	}
-
-	// Watch for changes to secondary resource Service
-	err = c.Watch(&source.Kind{Type: &corev1.Service{}}, &handler.EnqueueRequestForOwner{
-		IsController: true,
-		OwnerType:    &sriovnetworkv1.SriovNetworkNodePolicy{},
-	})
-
-	if err != nil {
-		return err
-	}
-
-	// Watch for changes to secondary resource Webhook
-	err = c.Watch(&source.Kind{Type: &admissionregistrationv1beta1.MutatingWebhookConfiguration{}}, &handler.EnqueueRequestForOwner{
-		IsController: true,
-		OwnerType:    &sriovnetworkv1.SriovNetworkNodePolicy{},
-	})
-
-	if err != nil {
-		return err
-	}
-
 	return nil
 }
 
@@ -179,7 +122,7 @@ func (r *ReconcileSriovNetworkNodePolicy) Reconcile(request reconcile.Request) (
 			// Default policy object not found, create it.
 			defaultPolicy.SetNamespace(Namespace)
 			defaultPolicy.SetName(DEFAULT_POLICY_NAME)
-			defaultPolicy.Spec =sriovnetworkv1.SriovNetworkNodePolicySpec{
+			defaultPolicy.Spec = sriovnetworkv1.SriovNetworkNodePolicySpec{
 				NumVfs:       0,
 				NodeSelector: make(map[string]string),
 				NicSelector:  sriovnetworkv1.SriovNetworkNicSelector{},
@@ -187,7 +130,7 @@ func (r *ReconcileSriovNetworkNodePolicy) Reconcile(request reconcile.Request) (
 			err = r.client.Create(context.TODO(), defaultPolicy)
 			if err != nil {
 				reqLogger.Error(err, "Failed to create default Policy", "Namespace", Namespace, "Name", DEFAULT_POLICY_NAME)
-				return reconcile.Result{},err
+				return reconcile.Result{}, err
 			}
 			return reconcile.Result{}, nil
 		}
@@ -197,7 +140,7 @@ func (r *ReconcileSriovNetworkNodePolicy) Reconcile(request reconcile.Request) (
 
 	// Fetch the SriovNetworkNodePolicyList
 	policyList := &sriovnetworkv1.SriovNetworkNodePolicyList{}
-	err = r.client.List(context.TODO(), &client.ListOptions{}, policyList)
+	err = r.client.List(context.TODO(), policyList, &client.ListOptions{})
 	if err != nil {
 		if errors.IsNotFound(err) {
 			// Request object not found, could have been deleted after reconcile request.
@@ -210,11 +153,22 @@ func (r *ReconcileSriovNetworkNodePolicy) Reconcile(request reconcile.Request) (
 	}
 	// Fetch the Nodes
 	nodeList := &corev1.NodeList{}
-	lo := &client.ListOptions{}
-	lbl := make(map[string]string)
-	lbl["node-role.kubernetes.io/worker"] = ""
-	lo.MatchingLabels(lbl)
-	err = r.client.List(context.TODO(), lo, nodeList)
+	lo := &client.MatchingLabels{}
+	defaultOpConf := &sriovnetworkv1.SriovOperatorConfig{}
+	if err := r.client.Get(context.TODO(), types.NamespacedName{Namespace: Namespace, Name: sriovoperatorconfig.DEFAULT_CONFIG_NAME}, defaultOpConf); err != nil {
+		return reconcile.Result{}, err
+	}
+	if len(defaultOpConf.Spec.ConfigDaemonNodeSelector) > 0 {
+		labels := client.MatchingLabels(defaultOpConf.Spec.ConfigDaemonNodeSelector)
+		lo = &labels
+
+	} else {
+		lo = &client.MatchingLabels{
+			"node-role.kubernetes.io/worker": "",
+			"beta.kubernetes.io/os":          "linux",
+		}
+	}
+	err = r.client.List(context.TODO(), nodeList, lo)
 	if err != nil {
 		// Error reading the object - requeue the request.
 		reqLogger.Error(err, "Fail to list nodes")
@@ -223,57 +177,22 @@ func (r *ReconcileSriovNetworkNodePolicy) Reconcile(request reconcile.Request) (
 
 	// Sort the policies with priority, higher priority ones is applied later
 	sort.Sort(sriovnetworkv1.ByPriority(policyList.Items))
+	// Sync Sriov device plugin ConfigMap object
+	if err = r.syncDevicePluginConfigMap(policyList); err != nil {
+		return reconcile.Result{}, err
+	}
+	// Render and sync Daemon objects
+	if err = r.syncPluginDaemonObjs(defaultPolicy, policyList); err != nil {
+		return reconcile.Result{}, err
+	}
 	// Sync SriovNetworkNodeState objects
 	if err = r.syncAllSriovNetworkNodeStates(defaultPolicy, policyList, nodeList); err != nil {
 		return reconcile.Result{}, err
 	}
-	// Sync SriovNetworkConfigDaemon objects
-	if err = r.syncConfigDaemonSet(defaultPolicy); err != nil {
-		return reconcile.Result{}, err
-	}
-	if len(policyList.Items) > 1 {
-		// Sync Sriov device plugin ConfigMap object
-		if err = r.syncDevicePluginConfigMap(policyList); err != nil {
-			return reconcile.Result{}, err
-		}
-		// Render and sync Daemon objects
-		if err = r.syncPluginDaemonObjs(defaultPolicy, policyList); err != nil {
-			return reconcile.Result{}, err
-		}
-	}
-	// Render and sync Webhook objects
-	if err = r.syncWebhookObjs(defaultPolicy); err != nil {
-		return reconcile.Result{}, err
-	}
 
 	// All was successful. Request that this be re-triggered after ResyncPeriod,
 	// so we can reconcile state again.
-	return reconcile.Result{}, nil
-}
-func (r *ReconcileSriovNetworkNodePolicy) syncConfigDaemonSet(dp *sriovnetworkv1.SriovNetworkNodePolicy) error {
-	logger := log.WithName("syncConfigDaemonset")
-	logger.Info("Start to sync config daemonset")
-	// var err error
-	objs := []*uns.Unstructured{}
-
-	data := render.MakeRenderData()
-	data.Data["Image"] = os.Getenv("SRIOV_NETWORK_CONFIG_DAEMON_IMAGE")
-	data.Data["Namespace"] = os.Getenv("NAMESPACE")
-	data.Data["ReleaseVersion"] = os.Getenv("RELEASEVERSION")
-	objs, err := renderDsForCR(DAEMON_PATH, &data)
-	if err != nil {
-		logger.Error(err, "Fail to render config daemon manifests")
-		return err
-	}
-	// Sync DaemonSets
-	for _, obj := range objs {
-		err = r.syncDsObject(dp, nil, obj)
-		if err != nil {
-			logger.Error(err, "Couldn't sync SR-IoV daemons objects")
-			return err
-		}
-	}
-	return nil
+	return reconcile.Result{RequeueAfter: ResyncPeriod}, nil
 }
 
 func (r *ReconcileSriovNetworkNodePolicy) syncDevicePluginConfigMap(pl *sriovnetworkv1.SriovNetworkNodePolicyList) error {
@@ -317,6 +236,10 @@ func (r *ReconcileSriovNetworkNodePolicy) syncDevicePluginConfigMap(pl *sriovnet
 	} else {
 		logger.Info("ConfigMap already exists, updating")
 		currentConfig := dptypes.ResourceConfList{}
+		err = json.Unmarshal([]byte(string(found.Data[DP_CONFIG_FILENAME])), &currentConfig)
+		if err != nil {
+			return err
+		}
 		if reflect.DeepEqual(currentConfig, data) {
 			logger.Info("No content change, skip update")
 			return nil
@@ -336,20 +259,45 @@ func (r *ReconcileSriovNetworkNodePolicy) syncAllSriovNetworkNodeStates(np *srio
 	if err := r.client.Get(context.TODO(), types.NamespacedName{Namespace: Namespace, Name: CONFIGMAP_NAME}, found); err != nil {
 		logger.Info("Fail to get", "ConfigMap", CONFIGMAP_NAME)
 	}
-	md5sum := md5.Sum([]byte(found.Data[DP_CONFIG_FILENAME]))
 	for _, node := range nl.Items {
 		logger.Info("Sync SriovNetworkNodeState CR", "name", node.Name)
 		ns := &sriovnetworkv1.SriovNetworkNodeState{}
 		ns.Name = node.Name
 		ns.Namespace = Namespace
 		j, _ := json.Marshal(ns)
-		fmt.Printf("SriovNetworkNodeState:\n%s\n\n", j)
-		if err := r.syncSriovNetworkNodeState(np, npl, ns, &node, hex.EncodeToString(md5sum[:])); err != nil {
+		logger.Info("SriovNetworkNodeState CR", "content", j)
+		if err := r.syncSriovNetworkNodeState(np, npl, ns, &node, found.GetResourceVersion()); err != nil {
 			logger.Error(err, "Fail to sync", "SriovNetworkNodeState", ns.Name)
 			return err
 		}
 	}
-
+	logger.Info("Remove SriovNetworkNodeState custom resource for unselected node")
+	nsList := &sriovnetworkv1.SriovNetworkNodeStateList{}
+	err := r.client.List(context.TODO(), nsList, &client.ListOptions{})
+	if err != nil {
+		if !errors.IsNotFound(err) {
+			logger.Info("Fail to list SriovNetworkNodeState CRs")
+			return err
+		}
+	} else {
+		for _, ns := range nsList.Items {
+			found := false
+			for _, node := range nl.Items {
+				logger.Info("validate", "SriovNetworkNodeState", ns.GetName(), "node", node.GetName())
+				if ns.GetName() == node.GetName() {
+					found = true
+					break
+				}
+			}
+			if !found {
+				err := r.client.Delete(context.TODO(), &ns, &client.DeleteOptions{})
+				if err != nil {
+					logger.Info("Fail to Delete", "SriovNetworkNodeState CR:", ns.GetName())
+					return err
+				}
+			}
+		}
+	}
 	return nil
 }
 
@@ -425,8 +373,30 @@ func (r *ReconcileSriovNetworkNodePolicy) syncPluginDaemonObjs(dp *sriovnetworkv
 		logger.Error(err, "Fail to render SR-IoV manifests")
 		return err
 	}
+
+	defaultConfig := &sriovnetworkv1.SriovOperatorConfig{}
+	err = r.client.Get(context.TODO(), types.NamespacedName{
+		Name: sriovoperatorconfig.DEFAULT_CONFIG_NAME, Namespace: Namespace}, defaultConfig)
+	if err != nil {
+		return err
+	}
 	// Sync DaemonSets
 	for _, obj := range objs {
+		if obj.GetKind() == "DaemonSet" && len(defaultConfig.Spec.ConfigDaemonNodeSelector) > 0 {
+			scheme := kscheme.Scheme
+			ds := &appsv1.DaemonSet{}
+			err = scheme.Convert(obj, ds, nil)
+			if err != nil {
+				logger.Error(err, "Fail to convert to DaemonSet")
+				return err
+			}
+			ds.Spec.Template.Spec.NodeSelector = defaultConfig.Spec.ConfigDaemonNodeSelector
+			err = scheme.Convert(ds, obj, nil)
+			if err != nil {
+				logger.Error(err, "Fail to convert to Unstructured")
+				return err
+			}
+		}
 		err = r.syncDsObject(dp, pl, obj)
 		if err != nil {
 			logger.Error(err, "Couldn't sync SR-IoV daemons objects")
@@ -436,40 +406,6 @@ func (r *ReconcileSriovNetworkNodePolicy) syncPluginDaemonObjs(dp *sriovnetworkv
 	return nil
 }
 
-func (r *ReconcileSriovNetworkNodePolicy) syncWebhookObjs(dp *sriovnetworkv1.SriovNetworkNodePolicy) error {
-	logger := log.WithName("syncWebhookObjs")
-	logger.Info("Start to sync webhook objects")
-
-	enable := os.Getenv("ENABLE_ADMISSION_CONTROLLER")
-	if enable != "true" {
-		logger.Info("SR-IOV Admission Controller is disabled, existing.")
-		logger.Info("To enable SR-IOV Admission Controller, set 'ENABLE_ADMISSION_CONTROLLER' to 'true'.")
-		return nil
-	}
-
-	// Render Webhook manifests
-	data := render.MakeRenderData()
-	data.Data["Namespace"] = os.Getenv("NAMESPACE")
-	data.Data["ServiceCAConfigMap"] = SERVICE_CA_CONFIGMAP
-	data.Data["SRIOVMutatingWebhookName"] = SRIOV_MUTATING_WEBHOOK_NAME
-	data.Data["NetworkResourcesInjectorImage"] = os.Getenv("NETWORK_RESOURCES_INJECTOR_IMAGE")
-	data.Data["ReleaseVersion"] = os.Getenv("RELEASEVERSION")
-	objs, err := render.RenderDir(WEBHOOK_PATH, &data)
-	if err != nil {
-		logger.Error(err, "Fail to render webhook manifests")
-		return err
-	}
-	// Sync Webhook
-	for _, obj := range objs {
-		err = r.syncWebhookObject(dp, obj)
-		if err != nil {
-			logger.Error(err, "Couldn't sync webhook objects")
-			return err
-		}
-	}
-	return nil
-}
-
 func (r *ReconcileSriovNetworkNodePolicy) tryDeleteDs(namespace, name string) error {
 	logger := log.WithName("tryDeleteDsObject")
 	ds := &appsv1.DaemonSet{}
@@ -517,139 +453,6 @@ func (r *ReconcileSriovNetworkNodePolicy) syncDsObject(dp *sriovnetworkv1.SriovN
 	return nil
 }
 
-func (r *ReconcileSriovNetworkNodePolicy) syncWebhookObject(dp *sriovnetworkv1.SriovNetworkNodePolicy, obj *uns.Unstructured) error {
-	var err error
-	logger := log.WithName("syncWebhookObject")
-	logger.Info("Start to sync Objects")
-	scheme := kscheme.Scheme
-	switch kind := obj.GetKind(); kind {
-	case "MutatingWebhookConfiguration":
-		whs := &admissionregistrationv1beta1.MutatingWebhookConfiguration{}
-		err = scheme.Convert(obj, whs, nil)
-		r.syncWebhook(dp, whs)
-		if err != nil {
-			logger.Error(err, "Fail to sync mutate webhook")
-			return err
-		}
-	case "ConfigMap":
-		cm := &corev1.ConfigMap{}
-		err = scheme.Convert(obj, cm, nil)
-		r.syncWebhookConfigMap(dp, cm)
-		if err != nil {
-			logger.Error(err, "Fail to sync webhook config map")
-			return err
-		}
-	case "ServiceAccount":
-		sa := &corev1.ServiceAccount{}
-		err = scheme.Convert(obj, sa, nil)
-		r.syncServiceAccount(dp, sa)
-		if err != nil {
-			logger.Error(err, "Fail to sync ServiceAccount")
-			return err
-		}
-	case "DaemonSet":
-		ds := &appsv1.DaemonSet{}
-		err = scheme.Convert(obj, ds, nil)
-		r.syncWebhookDaemonSet(dp, ds)
-		if err != nil {
-			logger.Error(err, "Fail to sync DaemonSet", "Namespace", ds.Namespace, "Name", ds.Name)
-			return err
-		}
-	case "Service":
-		s := &corev1.Service{}
-		err = scheme.Convert(obj, s, nil)
-		r.syncService(dp, s)
-		if err != nil {
-			logger.Error(err, "Fail to sync Service", "Namespace", s.Namespace, "Name", s.Name)
-			return err
-		}
-	case "ClusterRole":
-		cr := &rbacv1.ClusterRole{}
-		err = scheme.Convert(obj, cr, nil)
-		r.syncClusterRole(dp, cr)
-		if err != nil {
-			logger.Error(err, "Fail to sync cluster role", "Namespace", cr.Namespace, "Name", cr.Name)
-			return err
-		}
-	case "ClusterRoleBinding":
-		crb := &rbacv1.ClusterRoleBinding{}
-		err = scheme.Convert(obj, crb, nil)
-		r.syncClusterRoleBinding(dp, crb)
-		if err != nil {
-			logger.Error(err, "Fail to sync cluster role binding", "Namespace", crb.Namespace, "Name", crb.Name)
-			return err
-		}
-	}
-	return nil
-}
-
-func (r *ReconcileSriovNetworkNodePolicy) syncWebhook(cr *sriovnetworkv1.SriovNetworkNodePolicy, in *admissionregistrationv1beta1.MutatingWebhookConfiguration) error {
-	logger := log.WithName("syncWebhook")
-	logger.Info("Start to sync webhook", "Name", in.Name, "Namespace", in.Namespace)
-
-	if err := controllerutil.SetControllerReference(cr, in, r.scheme); err != nil {
-		return err
-	}
-	whs := &admissionregistrationv1beta1.MutatingWebhookConfiguration{}
-	err := r.client.Get(context.TODO(), types.NamespacedName{Name: in.Name}, whs)
-	if err != nil {
-		if errors.IsNotFound(err) {
-			err = r.client.Create(context.TODO(), in)
-			if err != nil {
-				return fmt.Errorf("Couldn't create webhook: %v", err)
-			}
-			logger.Info("Create webhook for", in.Namespace, in.Name)
-		} else {
-			return fmt.Errorf("Fail to get webhook: %v", err)
-		}
-	} else {
-		logger.Info("Webhook already exists, updating")
-		for idx, wh := range whs.Webhooks {
-			if wh.ClientConfig.CABundle != nil {
-				in.Webhooks[idx].ClientConfig.CABundle = wh.ClientConfig.CABundle
-			}
-		}
-		err = r.client.Update(context.TODO(), in)
-		if err != nil {
-			return fmt.Errorf("Couldn't update webhook: %v", err)
-		}
-	}
-	return nil
-}
-
-func (r *ReconcileSriovNetworkNodePolicy) syncWebhookConfigMap(cr *sriovnetworkv1.SriovNetworkNodePolicy, in *corev1.ConfigMap) error {
-	logger := log.WithName("syncWebhookConfigMap")
-	logger.Info("Start to sync config map", "Name", in.Name, "Namespace", in.Namespace)
-
-	if err := controllerutil.SetControllerReference(cr, in, r.scheme); err != nil {
-		return err
-	}
-	cm := &corev1.ConfigMap{}
-	err := r.client.Get(context.TODO(), types.NamespacedName{Namespace: in.Namespace, Name: in.Name}, cm)
-	if err != nil {
-		if errors.IsNotFound(err) {
-			err = r.client.Create(context.TODO(), in)
-			if err != nil {
-				return fmt.Errorf("Couldn't create config map: %v", err)
-			}
-			logger.Info("Create config map for", in.Namespace, in.Name)
-		} else {
-			return fmt.Errorf("Fail to get config map: %v", err)
-		}
-	} else {
-		logger.Info("Config map already exists, updating")
-		_, ok := cm.Data["service-ca.crt"]
-		if ok {
-			in.Data = cm.Data
-		}
-		err = r.client.Update(context.TODO(), in)
-		if err != nil {
-			return fmt.Errorf("Couldn't update config map: %v", err)
-		}
-	}
-	return nil
-}
-
 func (r *ReconcileSriovNetworkNodePolicy) syncService(cr *sriovnetworkv1.SriovNetworkNodePolicy, in *corev1.Service) error {
 	logger := log.WithName("syncService")
 	logger.Info("Start to sync service", "Name", in.Name, "Namespace", in.Namespace)
@@ -737,35 +540,6 @@ func (r *ReconcileSriovNetworkNodePolicy) syncClusterRoleBinding(cr *sriovnetwor
 	return nil
 }
 
-func (r *ReconcileSriovNetworkNodePolicy) syncWebhookDaemonSet(cr *sriovnetworkv1.SriovNetworkNodePolicy, in *appsv1.DaemonSet) error {
-	logger := log.WithName("syncWebhookDaemonSet")
-	logger.Info("Start to sync webhook daemon", "Name", in.Name, "Namespace", in.Namespace)
-
-	if err := controllerutil.SetControllerReference(cr, in, r.scheme); err != nil {
-		return err
-	}
-	ds := &appsv1.DaemonSet{}
-	err := r.client.Get(context.TODO(), types.NamespacedName{Namespace: in.Namespace, Name: in.Name}, ds)
-	if err != nil {
-		if errors.IsNotFound(err) {
-			err = r.client.Create(context.TODO(), in)
-			if err != nil {
-				return fmt.Errorf("Couldn't create webhook daemon: %v", err)
-			}
-			logger.Info("Create webhook daemon for", in.Namespace, in.Name)
-		} else {
-			return fmt.Errorf("Fail to webhook daemon: %v", err)
-		}
-	} else {
-		logger.Info("Webhook daemon already exists, updating")
-		err = r.client.Update(context.TODO(), in)
-		if err != nil {
-			return fmt.Errorf("Couldn't update webhook daemon: %v", err)
-		}
-	}
-	return nil
-}
-
 func (r *ReconcileSriovNetworkNodePolicy) syncServiceAccount(cr *sriovnetworkv1.SriovNetworkNodePolicy, in *corev1.ServiceAccount) error {
 	logger := log.WithName("syncServiceAccount")
 	logger.Info("Start to sync ServiceAccount", "Name", in.Name, "Namespace", in.Namespace)
@@ -890,8 +664,17 @@ func renderDevicePluginConfigData(pl *sriovnetworkv1.SriovNetworkNodePolicyList)
 			if p.Spec.NicSelector.Vendor != "" && !sriovnetworkv1.StringInArray(p.Spec.NicSelector.Vendor, rcl.ResourceList[i].Selectors.Vendors) {
 				rcl.ResourceList[i].Selectors.Vendors = append(rcl.ResourceList[i].Selectors.Vendors, p.Spec.NicSelector.Vendor)
 			}
-			if p.Spec.NicSelector.DeviceID != "" && !sriovnetworkv1.StringInArray(p.Spec.NicSelector.DeviceID, rcl.ResourceList[i].Selectors.Devices) {
-				rcl.ResourceList[i].Selectors.Devices = append(rcl.ResourceList[i].Selectors.Devices, p.Spec.NicSelector.DeviceID)
+			if p.Spec.NicSelector.DeviceID != "" {
+				var deviceID string
+				if p.Spec.NumVfs == 0 {
+					deviceID = p.Spec.NicSelector.DeviceID
+				} else {
+					deviceID = sriovnetworkv1.SriovPfVfMap[p.Spec.NicSelector.DeviceID]
+				}
+
+				if !sriovnetworkv1.StringInArray(p.Spec.NicSelector.DeviceID, rcl.ResourceList[i].Selectors.Devices) {
+					rcl.ResourceList[i].Selectors.Devices = append(rcl.ResourceList[i].Selectors.Devices, deviceID)
+				}
 			}
 			if len(p.Spec.NicSelector.PfNames) > 0 {
 				rcl.ResourceList[i].Selectors.PfNames = sriovnetworkv1.UniqueAppend(rcl.ResourceList[i].Selectors.PfNames, p.Spec.NicSelector.PfNames...)
@@ -916,7 +699,11 @@ func renderDevicePluginConfigData(pl *sriovnetworkv1.SriovNetworkNodePolicyList)
 				rc.Selectors.Vendors = append(rc.Selectors.Vendors, p.Spec.NicSelector.Vendor)
 			}
 			if p.Spec.NicSelector.DeviceID != "" {
-				rc.Selectors.Devices = append(rc.Selectors.Devices, p.Spec.NicSelector.DeviceID)
+				if p.Spec.NumVfs == 0 {
+					rc.Selectors.Devices = append(rc.Selectors.Devices, p.Spec.NicSelector.DeviceID)
+				} else {
+					rc.Selectors.Devices = append(rc.Selectors.Devices, sriovnetworkv1.SriovPfVfMap[p.Spec.NicSelector.DeviceID])
+				}
 			}
 			if l := len(p.Spec.NicSelector.PfNames); l > 0 {
 				rc.Selectors.PfNames = append(rc.Selectors.PfNames, p.Spec.NicSelector.PfNames...)
